var itemKeyword = [  'gms', 'chart', 'rain', 'temp' ];
var targetItem = itemKeyword[ 0 ];
var isPlaying = false;
var backgroundLoaders = new Array();

var py_command = null;
var retrievingItem = null;

var animationController = null;

function DEBUG( message ) {
    document.getElementById( 'debugDiv' ).innerText = message;
}

function init() {
    for ( var i = 0; i < itemKeyword.length; i++ ) {
        backgroundLoaders[ itemKeyword[ i ] ] = null;
    }
    
    var img = document.getElementById( 'graph' );
    if ( img == null ) return;
    if ( window.widget ) {
        invokeBackgroundLoaders();
        var output = widget.system( "/usr/bin/python getchart.py gms -list", null ).outputString;
        if ( output && output.length > 0 ) {
            var uris = output.split( "\n" );
            for ( var i = uris.length - 1; i >= 0; i-- ) {
                var uri = uris[ i ];
                if ( uri == "" ) continue;
                if ( uri.indexOf( "file:///" ) == 0 ) {
                    showGraph( targetItem, uri );
                }
                break;
            }
        }
        loadGraph( targetItem );
    }
}

function invokeBackgroundLoaders() {
    var int_reload = 60000 * 30;//100000; // 60000 * 30;
    var wait_chart = 120000;//20000; // 120000 ;

    setTimeout( function() { loadChartWithInterval( 'gms', int_reload ); }, wait_chart );
    setTimeout( function() { loadChartWithInterval( 'chart', int_reload ); }, wait_chart * 2 );
    setTimeout( function() { loadChartWithInterval( 'rain', int_reload ); }, wait_chart * 3 );
    setTimeout( function() { loadChartWithInterval( 'temp', int_reload ); }, wait_chart * 4 );

}


function loadChartWithInterval( item, duration ) {
    DEBUG( "set interval " + item  + " " + duration + " in loadChartWithInterval" );
    setInterval( function(){ loadGraph( item ); }, duration );
}

// load graph if no external script are running
function loadGraph( item ) {
    if ( py_command != null ) {
        DEBUG( 'already retrieving ' + retrievingItem + ", then loading '" + item + "' must wait for completion" );
        return;
    }
    
    var command = "/usr/bin/python getchart.py " + item;
    if ( item == targetItem ) {
        document.getElementById( 'graph' ).style.opacity = '0.5';
        document.getElementById( 'messageText' ).innerHTML = "Loading";
    }
    DEBUG( command );
    py_command = widget.system( command, newGraphHandler );
    retrievingItem = item.substring( 0, item.length );
}

// cancel current process and load
function forceLoadGraph( item ) {
    if ( py_command != null ) {
        if ( item == retrievingItem ) return;
        py_command.cancel();
    }
    loadGraph( item );
}

// process new graph
function newGraphHandler() {
    if ( py_command == null ) {
        DEBUG( 'no command' );
        return;
    }
    var output = py_command.outputString;
    var error = py_command.errorString;
//    py_command = null;
    if ( output ) {
        var outputs = output.split( "\n" );
        var uri = outputs[ 0 ];
        if ( retrievingItem == targetItem ) {
            showGraph( targetItem, uri );//, targetItem );
        } else {
            DEBUG( "store '" + uri + "' to backyard : " + targetItem + '/' + retrievingItem );
        }
    } else {
        DEBUG( error );
    }
    py_command = null;
    retrievingItem = null;
}





function getValidURIs( item ) {
    var command = "/usr/bin/python getchart.py " + item + " -list";
    var output = widget.system( command, null ).outputString;
    if ( output == undefined || output == "" ) {
        return new Array();
    }
    var URIs = output.split( "\n" );
    var validURIs = new Array();
    for ( var i = 0; i < URIs.length; i++ ) {
        if ( URIs[ i ].indexOf( 'file:///' ) == 0 ) {
            DEBUG( validURIs.length + ' ' + URIs[ i ] );
            validURIs.push( URIs[ i ] );
        }
    }        
    return validURIs;
}

function resolveItemName( item ) {
    if ( item == 'gms' ) {
        return "衛星写真";
    } else if ( item == 'chart' ) {
        return "天気図";
    } else if ( item == 'rain' ) {
        return "降水量";
    } else if ( item == 'temp' ) {
        return "気温";
    } else {
        return "??";
    }
}

function getDateFromURI( uri ) {
    var ps = uri.lastIndexOf( '/' );
    var pp = uri.lastIndexOf( '.' );
    var year = "1900";
    var month = "00";
    var day = "00";
    var hour = "00";
    if ( ps == -1 ) ps = 0;
    if ( pp == -1 ) pp = uri.length;
    if ( pp - ps >= 11 ) {
        year = uri.substring( ps + 1, ps + 5 );
        month = uri.substring( ps + 5, ps + 7 );
        day = uri.substring( ps + 7, ps + 9 );
        hour = uri.substring( ps + 9, ps + 11 );
    }
    return [ year, month, day, hour ];
}

function changeGraph( item ) {
    if ( isPlaying ) {
        DEBUG( 'cannot change while animation' );
        return;
    }
    DEBUG( "chageGraph " + item );
    targetItem = item;
    forceLoadGraph( item );
    return;
}

function showErrorMessage() {
    document.getElementById( 'graph' ).src = "Images/Empty.png";
    document.getElementById( 'messageText' ).innerHTML = "エラー発生";
}

function showGraph( item, uri ) {
    DEBUG( 'showGraph ' + item + " " + uri );
    var itemName = resolveItemName( item );
    var date = getDateFromURI( uri );
    var graph = document.getElementById( 'graph' );
    
    graph.src = uri;
    graph.style.opacity = '1.0';
    DEBUG( uri );
    document.getElementById( 'messageText' ).innerHTML = itemName + "<br/>" + date[ 1 ] + "/" + date[ 2 ] + "/" + date[ 3 ];
}




///////////// animation
function playMouseDown( event ) {
    if ( isPlaying ) {
        DEBUG( "STOPPPING" );
        document.getElementById( 'playButton' ).src="Images/ButtonStop2.png";
        window.addEventListener( "mouseup", stopMouseUp, true );
        window.addEventListener( "mouseexit", revertStopButton, true );
        return;
    } else {
        document.getElementById( 'playButton' ).src="Images/ButtonPlay2.png";
        window.addEventListener( "mouseup", playMouseUp, true );
        window.addEventListener( "mouseexit", revertPlayButton, true );
    }
}

function revertPlayButton( event ) {
    document.getElementById( 'playButton' ).src="Images/ButtonPlay1.png";
    window.removeEventListener( "mouseup", playMouseUp, true );
    window.removeEventListener( "mouseexit", revertPlayButton, true );
    event.stopPropagation(); 
    event.preventDefault(); 
}

function revertStopButton( event ) {
    document.getElementById( 'playButton' ).src="Images/ButtonStop1.png";
    window.removeEventListener( "mouseup", stopMouseUp, true );
    window.removeEventListener( "mouseexit", revertStopButton, true );
    event.stopPropagation(); 
    event.preventDefault(); 
}


function AnimationFrame( item, uris ) {
    this.item  = item;
    this.uris  = uris;
    this.count = 0;
    this.tick  = AnimationFrame__tick;
    this.timer = null;
}

function AnimationFrame__tick() {
    DEBUG( 'ANIMATION START ' + this.count + " " + this.uris.length + " " + this.uris[ this.count ] );
    showGraph( this.item, this.uris[ this.count ] );
    this.count ++;
    if ( this.count >= this.uris.length ) {
        if ( this.timer ) {
            DEBUG( 'stop' );
            clearInterval( this.timer );
            this.timer = null;
            isPlaying = false;
            animationController = null;
            document.getElementById( 'playButton' ).src="Images/ButtonPlay1.png";
        } else {
            DEBUG( "continueing " + this.count );
        }
    }
}

function playMouseUp( event ) {
    revertPlayButton( event );
    DEBUG( 'start animation' );
    //return;
    
    if ( window.widget ) {
        var item = targetItem;
        if ( item == undefined || item == null ) {
            item = 'gms';
        }
        var URIs = getValidURIs( item );
        if ( URIs.length > 0 ) {
            var interval = 250;
//             if ( targetItem == 'chart' ) {
//                 interval = 666;
//             }
            document.getElementById( 'playButton' ).src="Images/ButtonStop1.png";
            animationController = new AnimationFrame( targetItem, URIs );
            isPlaying = true;
            animationController.timer 
                = setInterval( function(){ animationController.tick(); }, interval );
        }
    }
}

function stopAnimation() {
    DEBUG( 'stop animation' );
    if ( animationController != null && isPlaying == true ) {
        animationController.count = animationController.uris.length - 1;
        animationController.tick();
    } else {
        DEBUG( "NOT STOPPED " + isPlaying );
    }
}

function stopMouseUp( event ) {
    DEBUG( "STOPPING ....." );
    revertStopButton( event );
    stopAnimation();
}







//////////// key events

function _mouse_down( event, upHandler, bname, img ) {
    window.addEventListener( 'mouseup', upHandler, true );
    document.getElementById( bname ).src = img;
    event.stopPropagation(); 
    event.preventDefault(); 
}

function _mouse_up( event, upHandler, bname, img ) {
    window.removeEventListener( 'mouseup', upHandler, true );
    document.getElementById( bname ).src = img;
    event.stopPropagation(); 
    event.preventDefault(); 
}

function gmsMouseDown( event ) {
    if ( isPlaying ) return;
    _mouse_down( event, gmsMouseUp, "gmsButton", "Images/ButtonGMS2.png" );
}

function gmsMouseUp( event ) {
    _mouse_up( event, gmsMouseUp, "gmsButton", "Images/ButtonGMS1.png" );
    changeGraph( 'gms' );
}


function chartMouseDown( event ) {
    if ( isPlaying ) return;
    _mouse_down( event, chartMouseUp, "chartButton", "Images/ButtonChart2.png" );
}

function chartMouseUp( event ) {
    _mouse_up( event, chartMouseUp, "chartButton", "Images/ButtonChart1.png" );
    changeGraph( 'chart' );
}

function rainMouseDown( event ) {
    if ( isPlaying ) return;
    _mouse_down( event, rainMouseUp, "rainButton", "Images/ButtonRain2.png" );
}

function rainMouseUp( event ) {
    _mouse_up( event, rainMouseUp, "rainButton", "Images/ButtonRain1.png" );
    changeGraph( 'rain' );
}

function tempMouseDown( event ) {
    if ( isPlaying ) return;
    _mouse_down( event, tempMouseUp, "tempButton", "Images/ButtonTemp2.png" );
}

function tempMouseUp( event ) {
    _mouse_up( event, tempMouseUp, "tempButton", "Images/ButtonTemp1.png" );
    changeGraph( 'temp' );
}
